/* IRCfs - IRC FileServ for *nix.
 * Copyright (C) 2002  Nick 'Zaf' Clifford <zaf@nrc.co.nz>
 * For licensing details, refer to the LICENSE file in the source 
 * code directory.
 */
#ifndef __include_list_h__
#define __include_list_h__

#include <unistd.h>
#include <ctype.h>
#include <sys/types.h>

struct list;

typedef int (*list_callback_t)(struct list *,void *contents, void *userdata); 
typedef void (*list_simple_callback_t)(void *data);
typedef int (*list_compare_t)(void *what, void *node);
typedef int (*list_delete_func_t)(struct list *, void *);

struct list *list_create(POOL p);
int list_destroy(struct list *list);

/* Sets the list delete node function.
 * This function is called for each node that is marked for deletion
 * when the list_cleanup() function is called. The object is for
 * the function to free any associated data with the node.
 * If the function returns -1, the cleanup is stopped
 */
int list_delete_func(struct list *list, list_delete_func_t callback);

/* Calls the list delete function for every node that has been marked
 * for deletion (by a call to the list_mark_current() function.
 */
int list_cleanup(struct list *list);

/* Marks the current node for deletion when the list_cleanup function
 * is called.
 */
int list_mark_current(struct list *list);

/* Calls callback for each item in the list.
 * Just before calling callback, sets the current pointer to that
 * node.
 */
int list_foreach(struct list *list, list_callback_t callback, void *userdata);

/* Simplier version of foreach, the callback is much simplier.
 */
int list_foreach_simple(struct list *list, list_simple_callback_t callback);


/* Adds an item to the beginning of the list.
 * Sets current pointer
 */
int list_add_front(struct list *list, void *contents);
/* Adds an item to the end of the list
 * Sets current pointer
 */
int list_add_end(struct list *list, void *contents);

/* finds an item in the list, calling comp_func(what,item) for each
 * item in the list until comp_func returns 0, then returns item.
 * returns NULL if not found.
 * Sets current pointer
 */
void *list_find(struct list *list, void *what, list_compare_t comp_func);

void *list_find_data(struct list *list, void *data);

/* Deletes the node at the current pointer. (Set with list_foreach or
 * list_find).
 * Sets current pointer to NULL afterwards.
 * Returns -1 if current pointer was already NULL.
 */
int list_del_current(struct list *list);

void *list_pop_first(struct list *list);
void *list_pop_last(struct list *list);
void *list_get_first(struct list *list);

int list_count(struct list *list);
int list_isempty(struct list *list);

#endif
